#!/usr/bin/env python
#encoding: utf8

import re
import sys
import subprocess
import os
import csv
import time
import select

def output_result_html(results):
    def result_to_line(res):
        (n, expected, counter_example) = res[:3]
        ccc = [(res[i*2+3], res[i*2+3+1]) for i in range(1)]
        cp = [(res[i*2+5], res[i*2+5+1]) for i in range(1)]
#        cpcp = [(res[i*2+11], res[i*2+11+1]) for i in range(2)]
        counter_example = {'False': False, 'True': True}[counter_example]
        def result_text(x):
            if x == 'equivalent': return '✓'
            elif x == 'nonequivalent': return '✗'
            elif x == 'timeout': return '⌛'
            else: return x
        def result_class(x, ref, has_counter_example=False):
            if has_counter_example:
                return 'unsound'
            elif x == 'timeout':
                return 'timeout'
            elif x == ref:
                return 'sound'
            elif x == 'equivalent':
                return 'unsound'
            else:
                return 'unprecise'
        def counter_example_text(l, ref, has_counter_example=False):
            if any((x, ref) == ("equivalent", "nonequivalent") for x in l) or not has_counter_example:
                return ''
            return '<br /><span class="warning">⚠ While <span class="tool">dizy</span>\'s conclusion is not unsound, the correlating program is!</span>'
        ccc_results = [{'TIME': t, 'TEXT': text, 'CHECK': result_text(text), 'CLASS': result_class(text, expected, counter_example)}
            for (text, t) in ccc]
        cp_results = [{'TIME': t, 'TEXT': text, 'CHECK': result_text(text), 'CLASS': result_class(text, expected)}
            for (text, t) in cp]
#        cpcp_results = [{'TIME': t, 'TEXT': text, 'CHECK': result_text(text), 'CLASS': result_class(text, expected)}
#            for (text, t) in cpcp]
        return """
<tbody>
<tr>
  <td rowspan="2">
    <a href="{INPUTIMP}.imp.html">{INPUTIMP}</a><br/>
    [<a href="patched.{INPUTIMP}.imp.html">patched</a>]
    [<a href="{INPUTIMP}.diffmap.html">syntactic diff</a>]
  </td>

  <td class="expectedresult">
    <span title="{EXPECTEDTEXT}" class="mark">{EXPECTEDRESULT}</span><br />
    ({EXPECTEDTEXT})
  </td>

  <td class="{CCC[0][CLASS]} cccresult">
    <span title="{CCC[0][TEXT]}" class="mark">{CCC[0][CHECK]}</span><br/>
    ({CCC[0][TIME]} seconds)
  </td>

  <td class="{CP[0][CLASS]} cccresult">
    <span title="{CP[0][TEXT]}" class="mark">{CP[0][CHECK]}</span><br/>
    ({CP[0][TIME]} seconds)
  </td>
</tr>
<tr>
  <td class="expectedtext">
  </td>

  <td class="unionlink">
    <a href="union.{INPUTIMP}.ccc.html">correlating program</a>
    {COUNTEREXAMPLE}
  </td>

  <td class="unionlink">
    <a href="union.{INPUTIMP}.cp.html">correlating program</a>
  </td>
</tr>

</tbody>""".format(INPUTIMP=n, EXPECTEDRESULT=result_text(expected),
                CCC=ccc_results,
                CP=cp_results,
                EXPECTEDTEXT=expected,
                COUNTEREXAMPLE=counter_example_text([c[0] for c in ccc], expected, counter_example))
    return """<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
  <head>
    <meta http-equiv="Content-Type" content="application/xhtml+xml; charset=utf-8"/>
    <title>Test results</title>
    <style type="text/css">
      table {{ width: 100%; border-collapse: collapse; }}
      th {{ text-align: center; background-color: rgb(180, 180, 180) }}

      table, tbody, th, td {{ border: 1px solid black; }}
      tr {{ border: none }}

/*      .unsound {{ background-color: rgb(255, 100, 100) }}
      .sound {{ background-color: rgb(100, 220, 100) }}
      .timeout {{ background-color: rgb(220, 220, 220) }}

      tbody:hover .unsound {{ background-color: rgb(230, 80, 80) }}
      tbody:hover .sound {{ background-color: rgb(80, 190, 80) }}
      tbody:hover .timeout {{ background-color: rgb(200, 200, 200) }} */

      tbody:nth-child(odd) {{ background-color: rgb(240, 240, 240) }}
/*      tbody:hover {{ background-color: rgb(250, 250, 250) }} */

      .unsound .mark, .warning {{ color: rgb(240, 0, 0) }}
      .sound .mark {{ color: rgb(0, 200, 0) }}
      .timeout .mark {{ color: rgb(170, 170, 170) }}

      .cccresult, .cpresult, .cpcpresult {{ border-bottom: 1px dashed gray; width: 40% }}
      .cccresult, .cpresult, .cpcpresult, .expectedresult {{ text-align: center }}
      .expectedresult {{ border-bottom: none }}

      .mark {{ font-size: xx-large }}
      .unionlink, .expectedtext {{ border-top: none; text-align: center }}

      .tool {{ font-family: Monospace }}
      .lang {{ font-family: Monospace }}
    </style>
  </head>
  <body>
    <h1>Experiments</h1>
    <p>While the <em>soundness</em> of our <span class="tool"><a href="../CorrelatingProgram.tar.gz">correlating_program</a></span>
    is <a href="../coq/CorrelatingProgram.html#correlating_program_sound">proven in the <span class="tool">Coq</span> proof assistant</a>, we evaluate
    the <em>quality of its output</em> by comparing the results of its analysis
    by <span class="tool">dizy</span>.</p>

    <h2>Protocol</h2>

    <p>Our comparison is done on a selection of <a href="examples.tar.gz">difference examples</a>
    presented as pairs of programs in our source language. For each of
    those pairs:
      <ol>
        <li>A representation Δ of the syntactic differences between the two
            programs of the pair is computed using a simple recursive
            algorithm.</li>
        <li>The two input programs are translated to actual <span class="lang">C</span> for <span class="tool">ccc</span>.</li>
        <li>A first correlating program is generated from the two <span class="lang">C</span> programs using <span class="tool">ccc</span>.</li>
        <li>A second correlating program is generated from Δ using <span class="tool">correlating_program</span>.</li>
        <li><span class="tool">Dizy</span> is used to analyze both correlating programs, and its output is used to tell if the return value of each considered function may have changed (✗) or not (✓).</li>
      </ol>
    </p>

    <p>All of this is automated by a <span class="lang">Python</span> (<span class="tool">run_tests.py</span>) script
    that takes a file listing the tests along with the expected result
    (equivalent or non-equivalent), and an optional counter-example to the
    correction of the correlating program.</p>

    <h2>Results</h2>

    <table>
      <thead>
        <tr>
          <th>Example</th>
          <th>Expected result</th>
          <th abbr="ccc"><span class="tool">ccc</span> + <span class="tool">dizy</span></th>
          <th abbr="cp"><span class="tool">correlating_program</span> + <span class="tool">dizy</span></th>
        </tr>
      </thead>
      {TABLE}
    </table>
  </body>
</html>
""".format(TABLE=''.join(result_to_line(x) for x in results))


def main(path):
    with open(path, 'r') as in_file:
        reader = csv.reader(in_file)
        _ = next(reader)
        c = output_result_html(reader)
    with open('index.html', 'w') as file:
        file.write(c)


if __name__ == '__main__':
  main(*sys.argv[1:])
